﻿using System;
using System.Collections.Generic;
using System.Linq;
using JSLintForResharper.Infrastructure;
using JSLintForResharper.JSLint;
using JSLintForResharper.Resharper.Highlighting;
using JSLintForResharper.Utils;
using JetBrains.ReSharper.Daemon;
using JetBrains.ReSharper.Psi;
using JetBrains.ReSharper.Psi.JavaScript.LanguageImpl;
using JetBrains.ReSharper.Psi.JavaScript.Tree;
using JetBrains.ReSharper.Psi.Tree;

namespace JSLintForResharper.Resharper.Deamon
{
	internal class DaemonStageProcess : IDaemonStageProcess
	{
		private readonly IDaemonProcess daemonProcess;
		private readonly List<HighlightingInfo> highlightings = new List<HighlightingInfo>();
		private IJavaScriptFile mainFile;
		private IJavaScriptTreeNode mainNode;
		private List<HighlightedWarning> warnings;
		private string code;
		private CoordinateToOffsetTranslator offsetTranslator;
		private int tabSize = 4;
		private bool isMixed;
		private int lineOffset;

		public DaemonStageProcess(IDaemonProcess daemonProcess)
		{
			this.daemonProcess = daemonProcess;
		}

		public IDaemonProcess DaemonProcess
		{
			get { return daemonProcess; }
		}

		public void Execute(Action<DaemonStageResult> commiter)
		{
			if (!daemonProcess.FullRehighlightingRequired)
				return;

			var sourceFile = daemonProcess.SourceFile;

#warning Changed here
			mainFile = sourceFile.GetTheOnlyPsiFile(JavaScriptLanguage.Instance) as IJavaScriptFile;
			highlightings.Clear();

			if (mainFile != null)
			{
				if (sourceFile.PrimaryPsiLanguage is JavaScriptLanguage)
				{
					isMixed = false;
					mainNode = mainFile;
					LintAndAddHighlightings();
				}
				else
				{
					isMixed = true;
					foreach (var subFile in mainFile.Sections)
					{
						mainNode = subFile;
						LintAndAddHighlightings();
					}
				}
			} 
			
			commiter(new DaemonStageResult(highlightings));
		}

		private IEnumerable<LintWarning> Lint(string input)
		{
			bool addedWhite = false;
			if (isMixed && !Settings.Values.ContainsKey("white"))
			{
				Settings.Values.Add("white", true);
				addedWhite = true;
			}

			var settings = Settings.Values
				.Where(pair => pair.Value != null && !pair.Key.StartsWith("R#_"))
				.Select(pair => "'" + pair.Key + "':'" + pair.Value.ToString().ToLower() + "'")
				.ToArray();
			var options = "{" + String.Join(",", settings) + "}";

			if (isMixed && addedWhite)
				Settings.Values.Remove("white");

			var lintWarnings = ScriptRunner.Evaluate(input, options);
			return lintWarnings;
		}

		private void LintAndAddHighlightings()
		{
			try
			{
				lineOffset = 0;
				var sectionCode = mainNode.GetText();
				code = sectionCode;
				if (isMixed)
					AppendCommentsAndAdjustLineOffset();
				offsetTranslator = new CoordinateToOffsetTranslator(sectionCode, lineOffset);
				warnings = Lint(code).Select(CreateHighlightedWarning).ToList();

				foreach (var highlightedWarning in warnings)
				{
                    // Ghnn, really get how to do this right some day..
                    
                    var offset = new TreeOffset(highlightedWarning.Offset);
				    
				    var nodes = mainNode.FindNodesAt(offset)
                        .OrderBy(n => n is IWhitespaceNode ? 1 : 0)
                        .ToList();
				    var origToken = mainNode.FindTokenAt(offset);
                    var token = origToken;
                    while (token is IWhitespaceNode)
				        token = token.PrevSibling;
                    if (token != null && !nodes.Contains(token))
    				    nodes.Insert(0, token);
				    token = origToken;
                    while (token is IWhitespaceNode)
				        token = token.NextSibling;
                    if (token != null && !nodes.Contains(token))
    				    nodes.Insert(1, token);

                    if (IsWhitespaceWarning(highlightedWarning))
				        AddWhitespaceWarning(highlightedWarning, nodes);
				    else
                        TryAddWarningOnEvidence(highlightedWarning, nodes);

				    if (IsNotHightlighted(highlightedWarning, nodes))
                        AddNonWhitespaceWarning(highlightedWarning, nodes);
				}
			}
			catch (Exception ex)
			{
				Log(ex);
			}
		}

		private void AppendCommentsAndAdjustLineOffset()
		{
			var comments = mainNode.LeftSiblings().OfType<IJavaScriptCommentNode>()
				.Select(n => n.GetText().Replace("\r", "").Replace("\n", ""))
				.ToArray();
			lineOffset = comments.Length;
			var commentString = String.Join("\n", comments);
			if (lineOffset > 0)
				code = commentString + "\n" + code;
		}

		private void AddNonWhitespaceWarning(HighlightedWarning highlightedWarning, IEnumerable<ITreeNode> nodes)
		{
			var node = nodes.FirstOrDefault(n => !n.IsWhitespaceToken());
			AddHighlighting(highlightedWarning, node);
		}

		private void TryAddWarningOnEvidence(HighlightedWarning highlightedWarning, IList<ITreeNode> nodes)
		{
			for (var i = 0; i < nodes.Count; i++)
			{
				var node = nodes[i];
				ITreeNode parent = null;
				var evidence = highlightedWarning.Warning.Evidence;
				var nodeText = node.GetText();
				var noMatch = nodeText != evidence;
				while (noMatch && evidence.Contains(nodeText) && node != null)
                {
                    node = node.Parent;
                    if (node == null) continue;
                    nodeText = node.GetText();
                    noMatch = nodeText != evidence;
                }

                if (noMatch)
                    continue;

				AddHighlighting(highlightedWarning, node);
				break;
			}
		}

		private void AddWhitespaceWarning(HighlightedWarning highlightedWarning, IEnumerable<ITreeNode> nodes)
		{
			var node = nodes.FirstOrDefault(n => n.IsWhitespaceToken());
			if (node != null)
				AddHighlighting(highlightedWarning, node);
		}

		private static bool IsWhitespaceWarning(HighlightedWarning highlightedWarning)
		{
			return highlightedWarning.Warning.Message.ToLower().Contains("space");
		}

		private static bool IsNotHightlighted(HighlightedWarning highlightedWarning, IEnumerable<ITreeNode> nodes)
		{
			return !highlightedWarning.Highlighted && nodes.Any();
		}

		private HighlightedWarning CreateHighlightedWarning(LintWarning warning)
		{
			int offset = GetMessageOffset(warning);
			return new HighlightedWarning(warning, offset);
		}

		private void AddHighlighting(HighlightedWarning warning, ITreeNode node)
		{
			var sameHighlight = highlightings.SingleOrDefault(SameNode(node));
			if (sameHighlight != null)
			{
			    var existing = ((LintHighlightingBase)sameHighlight.Highlighting);
			    existing.AddWarning(warning.Warning);
			}
			else
			{
				var highlightingInfo = new HighlightingInfo(node.GetDocumentRange(), CreateHighlighting(warning, node));
				highlightings.Add(highlightingInfo);
			}
			warning.Highlighted = true;
		}

		private static LintHighlightingBase CreateHighlighting(HighlightedWarning warning, ITreeNode node)
		{
			if (Settings.Get<bool?>("R#_priority") == true)
				return new PrioritizedLintHighlighting(node, warning.Warning);
			return new LintHighlighting(node, warning.Warning);
		}

		private static Func<HighlightingInfo, bool> SameNode(ITreeNode node)
		{
		    return h =>
		    {
		        var existingRange = ((LintHighlightingBase) h.Highlighting).Expression.GetTreeTextRange();
		        var newRange = node.GetTreeTextRange();
		        return existingRange.Equals(newRange) || existingRange.Intersects(newRange);
		    };
		}

		private int GetMessageOffset(LintWarning message)
		{
			return offsetTranslator.GetOffset(message.Line, message.Character);
		}

		private void Log(Exception ex)
		{
			Log("", ex);
		}

		private void Log(string extra, Exception ex, params object[] args)
		{
			LogProvider.Instance.Log(String.Format("{0}: {1}, {2}" + extra,
				new object[]{
					DateTime.Now.ToString("yy-MM-dd HH:mm:ss"),
					ex.Message + Environment.NewLine + ex.StackTrace,
					daemonProcess.SourceFile.Name
				}.Union(args).ToArray()
			));
		}
	}

	public class HighlightedWarning
	{
		public LintWarning Warning { get; private set; }
		public int Offset { get; private set; }
		public bool Highlighted { get; set; }

		public HighlightedWarning(LintWarning warning, int offset)
		{
			Warning = warning;
			Offset = offset;
		}
	}
}
